{% load menu_tags %}

{% for menu_item in menu_items %}
    {% show_permitted_menu_entry menu_item %}
        {% if menu_item.active %}
            <li class="active">
                {% else %}
            <li>
        {% endif %}

    <a href="{% url menu_item.view %}">{{ menu_item.display_name }}</a>
    </li>
    {% endshow_permitted_menu_entry %}
{% endfor %}